//============================================================================
// Name        : Sudoku.cpp
// Author      : Frank,frank@zijinshi.cn
// Version     : V0.0.1
// Copyright   :
// Description : Sudoku in C++
//============================================================================

#include <iostream>
using namespace std;


const int COUNT = 9;
char oriArray[COUNT][COUNT];
const char* INPUT_FILE = "input.txt";
const char* OUTPUT_FILE = "output.txt";

void initialize();
char* readInput( );
void getResult( );
int meet(int i, int j, char value);
void iteratorArray( int i, int j );
int getNext( int &i, int &j);
void outputResult(int check);
int checkAll();

int main() {
	int resolved = 0;
	initialize();
	char *ret = readInput();
	if( strlen(ret) > 0 )
	{
		FILE *pFile;
		pFile = fopen( OUTPUT_FILE, "w" );
		fputs( ret, pFile );
		fclose( pFile );
		delete ret;
		return 0 ;
	}
	delete ret;
	getResult( );
	resolved = checkAll( );
	if( resolved == 0)
	{
		outputResult( 0 );
	}
	return 0;
}

void initialize()
{
	// remove output file if exists.
	remove( OUTPUT_FILE );
}

void getResult( )
{
	iteratorArray( 0, 0 );
}
/**
 * 1 can get; 0 end
 */
int getNext( int &i, int &j)
{
	if( i == 8 && j == 8 )
	{
		return 0;
	}
	if( j == 8 )
	{
		i ++;
		j = 0;
	}else
	{
		j ++;
	}

	return 1;
}

void iteratorArray( int i, int j )
{
	if( i == 9)
	{
		exit(0);
	}
	if( oriArray[i][j] == '0' )
	{

		for( int k = 0; k < 9; k ++)
		{
			char value = '0' + k + 1;
			oriArray[i][j] = value;
			if( meet( i, j, value ) == 1 )
			{
				int nextI = i;
				int nextJ = j;
				int hasNext = getNext( nextI, nextJ );
				if( hasNext == 0 )
				{
					// print result;
					int check = checkAll( );
					outputResult( check );
					exit(0);
				}
				iteratorArray( nextI, nextJ );
			}else
			{
//				if( i == 8 && j == 8)
//				{
//					int check = checkAll( );
//					outputResult( check );
//					exit( 0 );
//				}
				continue;
			}
		}
		oriArray[i][j] = '0';

	}else
	{
		int nextI = i;
		int nextJ = j;
		int hasNext = getNext( nextI, nextJ );
		if( hasNext == 0 )
		{
			// print result;
			int check = checkAll( );
			outputResult( check );
			exit(0);
		}
		iteratorArray( nextI, nextJ );
	}

}

/**
 * Double check. In fact this function is not needed.
 * It returns 0 when the original array is error, or no resolution.
 *  1 true, 0 false;
 */
int checkAll( )
{
	for( int i = 0; i < 9; i ++ )
		for( int j = 0; j < 9; j ++ )
		{
			if( meet( i, j, oriArray[i][j] ) == 0 )
			{
				return 0;
			}
		}
	return 1;
}

/**
 *  1 true, 0 false;
 */
int meet( int i, int j, char value )
{
	int k1;
	for( k1 = 0; k1 < 9; k1 ++ )
	{
		if( k1 != j && oriArray[i][k1] == value )
		{
			return 0;
		}
		if( k1 != i && oriArray[k1][j] == value )
		{
			return 0;
		}
	}

	int k2;
	int bI = i / 3;
	int bJ = j / 3;

	for( k1 = 0; k1 < 3; k1 ++ )
		for( k2 = 0; k2 < 3; k2 ++ )
		{
			if( bI * 3 + k1 == i || bJ * 3 + k2 == j  )
			{
				continue;
			}
			if( oriArray[ bI * 3 + k1 ][ bJ * 3 + k2] == value )
			{
				return 0;
			}
		}

	return 1;
}

char* readInput( )
{
	char* retString = new char[200];
	memset( retString, 0, 200 );
	FILE *pFile;
	pFile = fopen(INPUT_FILE,"rt");
	if( pFile == NULL )
	{
		strcat(retString, "ȡ" );
		strcat(retString, INPUT_FILE );
		strcat(retString, "ļ󣬴ļڰɣ" );
		return retString;
	}

	int iLineCnt = 0;
	char line[50];
	while( !feof(pFile) )
	{
		memset(line, 0, 50);
		fgets( line, 50, pFile);
		int iCnt = 0;

		for( int i = 0; i <= strlen( line ); i ++ )
		{
			if( line[i] == '\n')
			{
				break;
			}
			if( line[i] > '9' || line[i] < '0' )
			{
				continue;
			}
			oriArray[iLineCnt][iCnt] = line[i];
			iCnt ++;

			if( iCnt >= 9 )
			{
				break;
			}

		}
		if( iCnt < 9 )
		{
			fclose(pFile);
			strcat(retString, "ļȡŶ̫˰ɣһinput.txt" );
			retString[ strlen( retString ) ] = ( iLineCnt + 1 ) + '0';
			strcat(retString, "С^_^" );
			return retString;
		}
		iLineCnt ++;
		if( iLineCnt >= 9 )
		{
			break;
		}
	}
	if( iLineCnt < 9 )
	{
		fclose(pFile);

		strcat(retString, "ļȡŶСѣôֻ" );
		retString[ strlen( retString ) ] = ( iLineCnt + 1 ) + '0';
		strcat(retString, "?Źͼ99ô ^o^" );
		return retString;
	}
	fclose(pFile);
	return "";
}

void outputResult( int check )
{
	FILE *pFile;
	pFile = fopen( OUTPUT_FILE, "w" );
	if( check == 0 )
	{
		fputs( "޽⣡", pFile );
		return;
	}
	for( int i = 0; i < 9; i ++ )
	{
		for( int j = 0; j < 9; j ++ )
		{
			fputc( oriArray[i][j], pFile );
			fputc( ' ', pFile );
		}
		fputc( '\n', pFile );
	}
	fclose( pFile );
}
